#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: James Laska <jlaska@redhat.com>

import os
import os.path
import grp, stat
import re
import StringIO
import urlgrabber
import ConfigParser
import autoqa.util
from autoqa.repoinfo import repoinfo
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher
from autotest_lib.client.bin import utils
from autotest_lib.client.common_lib import error

class compose_tree(AutoQATest):
    version = 1 # increment if setup() changes

    @ExceptionCatcher()
    def setup(self, **kwargs):
        super(self.__class__, self).setup()
        utils.system('yum -y install mock')
        # If autotest isn't already a member of the mock group, add it now
        utils.system('usermod -a -G mock autotest')

    @ExceptionCatcher()
    def run_once(self, *args, **kwargs):
        super(self.__class__, self).run_once()

        assert kwargs['hook'] in ['post-koji-build',], \
            'Unexpected hook argument: %s' % kwargs['hook']
        assert kwargs.has_key('envr'), \
            'Missing required argument: envr'
        assert kwargs.has_key('kojitag'), \
            'Missing required argument: kojitag'

        # Assume success
        self.result = 'PASSED'

        # Determine releasever to use for compose_tree.sh script
        repo = repoinfo.getrepo_by_tag(kwargs.get('kojitag'))
        # FIXME - error check 'repo' value
        if repo['name'] != 'rawhide':
            # FIXME - shouldn't the "version" of a release be in repoinfo too?
            m = re.match(r'^f([0-9]+)-\b', repo['name'], re.IGNORECASE)
            releasever = 'fedora-%s' % m.group(1)
        else:
            releasever = 'fedora-rawhide'

        # Adjust permissions so autotest can save results
        os.chown(self.resultsdir, -1, grp.getgrnam('autotest').gr_gid) # set group = 'autotest'
        os.chmod(self.resultsdir, os.stat(self.resultsdir).st_mode | stat.S_IWGRP) # allow group write

        # Run test
        cmd = '%s/compose_tree.sh -r %s -d %s -e "%s" ' % (self.bindir, \
              releasever, self.resultsdir, kwargs.get('envr'))
        cmd = 'su -c "%s" - autotest' % cmd
        try:
            out = utils.system_output(cmd, retain_output=True)
        except error.CmdError, e:
            self.result = 'FAILED'
            out = e.result_obj.stdout
        finally:
            self.outputs = out

        # Summarize test results
        self.highlights += '%s\n' % cmd
        self.highlights += '\n== Highlights ==\n'

        for fname in ['boot.iso', 'vmlinuz', 'initrd.img']:
            if os.path.exists(os.path.join(self.resultsdir, fname)):
                self.highlights += " * %s - OK\n" % fname
            else:
                self.highlights += " * %s - MISSING\n" % fname
                self.result = 'FAILED'

